package UI_Converter.bytecode;

import java.io.ByteArrayOutputStream;
import org.xml.sax.Attributes;
import UI_Converter.bytecode.Attribute;
import UI_Converter.common.Tag;
import UI_Converter.util.Conversion;
import UI_Converter.util.Element;
import UI_Converter.util.FileManager;

/**
 * An ExecuteUSATCommand object stores the content of the execute-USAT-command
 * XML element and generates the corresponding ExecuteUSATCommand UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class ExecuteUSATCommand extends Element {
   /**
    * USAT command type.
    */
   private byte commandType;

   /**
    * USAT command qualifier.
    */
   private byte commandQualifier;

   /**
    * Destination device.
    */
   private byte destinationDevice;

   /**
    * Create a new ExecuteUSATCommand object.
    *
    * @param atts Attributes of the execute-USAT-command XML element.
    */
   public ExecuteUSATCommand(Attributes atts) {
      setTagValue(Tag.EXECUTE_USAT_COMMAND);

      boolean optimization = Conversion.yesNoStringToBoolean(atts.getValue("optimization"));
      if (!optimization) {
         setAttribute(Attribute.ExecuteUSATCommand.RESULT_OPTIMIZATION_REQUIRED);
      }

      String behaviourValue = atts.getValue("behaviour");
	  if (behaviourValue != null){
		if (behaviourValue.equals("continue-any-GR")) {
		  setAttribute(Attribute.ExecuteUSATCommand.BEHAVIOUR_OFFSET, Attribute.ExecuteUSATCommand.CONTINUE_ANY_GR);
		}
		else if (behaviourValue.equals("stop-GRvar-not-present")) {
		  setAttribute(Attribute.ExecuteUSATCommand.BEHAVIOUR_OFFSET, Attribute.ExecuteUSATCommand.STOP_GRVAR_NOT_PRESENT);
		}
		else if (behaviourValue.equals("stop-GRvar-not-present")) {
		  setAttribute(Attribute.ExecuteUSATCommand.BEHAVIOUR_OFFSET, Attribute.ExecuteUSATCommand.STOP_GRVAR_PRESENT);
		}
		else if (behaviourValue.equals("invokeTRH-any-GR")) {
		  setAttribute(Attribute.ExecuteUSATCommand.BEHAVIOUR_OFFSET, Attribute.ExecuteUSATCommand.INVOKETRH_ANY_GR);
		}
	  }

      commandType       = Conversion.hexaStringToByte(atts.getValue("cmd-type"));
      commandQualifier  = Conversion.hexaStringToByte(atts.getValue("cmd-qual"));
      destinationDevice = Conversion.hexaStringToByte(atts.getValue("dest-device"));
   }

   /**
    * Generate the byte code of the Execute USAT Command.
    *
    * @param buffer Buffer in which the byte code is generated.
    *
    * @return the number of generated bytes.
    */
	public int generate(ByteArrayOutputStream buffer) {
		int length = 0;
		int index = getChildrenCount() - 1;


		Element element = null;
		while (index >= 0) {
			element = getChild(index);
			if (!(element instanceof USATCommandTLV)) {
				break;
			}
			length += element.generate(buffer);
			index --;
		}

		buffer.write(destinationDevice);
		buffer.write(commandQualifier);
		buffer.write(commandType);
		length += 3;
		GeneralResultVariable generalResultVariable = null;
		if (index >= 0) {

			if (element instanceof OutputVariable) {
				OutputVariable outVar = (OutputVariable)element;
				setAttribute(Attribute.ExecuteUSATCommand.OUTPUT_VARIABLE);
				length += outVar.generate(buffer);
				index --;
			}
			if (index == 0) {
				element = getChild(index);
				GeneralResultVariable GRvar = (GeneralResultVariable)element;
				length += GRvar.generate(buffer);
			}
		}
		length += generateAttributes(buffer);
		length += generateBERTLVLength(length, buffer);
		length += generateTag(buffer);
		return length;
	}	
}